import logging
import typing

from flyteidl.core import artifact_id_pb2 as art_id
from flytekit.clients.auth_helper import (
    get_channel,
    upgrade_channel_to_authenticated,
    upgrade_channel_to_proxy_authenticated,
    wrap_exceptions_channel,
)
from flytekit.clients.friendly import SynchronousFlyteClient
from flytekit.configuration import Config
from flytekit.core.type_engine import TypeEngine
from flytekit.exceptions import user as user_exceptions
from flytekit.models.core.identifier import ResourceType
from flytekit.remote import FlyteRemote
from flytekit.remote.entities import FlyteLaunchPlan, FlyteTask, FlyteWorkflow
from flytekit.remote.executions import FlyteNodeExecution, FlyteTaskExecution, FlyteWorkflowExecution
from flytekit.tools.translator import (
    Options,
)

from unionai._config import _get_config_obj, _get_organization
from unionai._interceptor import update_client_with_interceptor
from unionai.artifacts import Artifact
from unionai.internal.artifacts import artifacts_pb2, artifacts_pb2_grpc


class UnionRemote(FlyteRemote):
    def __init__(
        self,
        config: typing.Optional[Config] = None,
        default_project: typing.Optional[str] = "default",
        default_domain: typing.Optional[str] = "development",
        data_upload_location: str = "flyte://my-s3-bucket/",
        **kwargs,
    ):
        if config is None:
            config = _get_config_obj(config, default_to_serverless=True)

        super().__init__(config, default_project, default_domain, data_upload_location, **kwargs)
        self._artifacts_client = None

    @property
    def client(self) -> SynchronousFlyteClient:
        """Return a SynchronousFlyteClient for additional operations."""
        if not self._client_initialized:
            client = SynchronousFlyteClient(self.config.platform, **self._kwargs)
            org = _get_organization(self.config.platform)
            self._client = update_client_with_interceptor(client, org)
            self._client_initialized = True

        return self._client

    def generate_console_url(
        self,
        entity: typing.Union[
            FlyteWorkflowExecution, FlyteNodeExecution, FlyteTaskExecution, FlyteWorkflow, FlyteTask, FlyteLaunchPlan
        ],
    ):
        """
        Generate a UnionAI console URL for the given Flyte remote endpoint.

        This will automatically determine if this is an execution or an entity
        and change the type automatically.
        """
        org = _get_organization(self.config.platform)
        if org == "":
            return super().generate_console_url(entity)

        if isinstance(entity, (FlyteWorkflowExecution, FlyteNodeExecution, FlyteTaskExecution)):
            return f"{self.generate_console_http_domain()}/org/{org}/projects/{entity.id.project}/domains/{entity.id.domain}/executions/{entity.id.name}"  # noqa

        if not isinstance(entity, (FlyteWorkflow, FlyteTask, FlyteLaunchPlan)):
            raise ValueError(f"Only remote entities can be looked at in the console, got type {type(entity)}")
        rt = "workflow"
        if entity.id.resource_type == ResourceType.TASK:
            rt = "task"
        elif entity.id.resource_type == ResourceType.LAUNCH_PLAN:
            rt = "launch_plan"
        return f"{self.generate_console_http_domain()}/org/{org}/projects/{entity.id.project}/domains/{entity.id.domain}/{rt}/{entity.name}/version/{entity.id.version}"  # noqa

    @property
    def artifacts_client(self) -> artifacts_pb2_grpc.ArtifactRegistryStub:
        if self._artifacts_client:
            return self._artifacts_client

        cfg = self.config.platform
        channel = wrap_exceptions_channel(
            cfg, upgrade_channel_to_authenticated(cfg, upgrade_channel_to_proxy_authenticated(cfg, get_channel(cfg)))
        )
        self._artifacts_client = artifacts_pb2_grpc.ArtifactRegistryStub(channel)

        return self._artifacts_client

    def get_artifact(
        self,
        uri: typing.Optional[str] = None,
        artifact_key: typing.Optional[art_id.ArtifactKey] = None,
        artifact_id: typing.Optional[art_id.ArtifactID] = None,
        query: typing.Optional[art_id.ArtifactQuery] = None,
        tag: typing.Optional[str] = None,
        get_details: bool = False,
    ) -> typing.Optional[Artifact]:
        if query:
            q = query
        elif uri:
            q = art_id.ArtifactQuery(uri=uri)
        elif artifact_key:
            if tag:
                q = art_id.ArtifactQuery(artifact_tag=art_id.ArtifactTag(artifact_key=artifact_key, tag=tag))
            else:
                q = art_id.ArtifactQuery(artifact_id=art_id.ArtifactID(artifact_key=artifact_key))
        elif artifact_id:
            if tag:
                raise ValueError("If using tag specify key instead of ID.")
            q = art_id.ArtifactQuery(artifact_id=artifact_id)
        else:
            raise ValueError("One of uri, key, id")
        req = artifacts_pb2.GetArtifactRequest(query=q, details=get_details)
        resp = self.artifacts_client.GetArtifact(req)
        a = Artifact.from_flyte_idl(resp.artifact)
        return a

    def _execute(
        self,
        entity: typing.Union[FlyteTask, FlyteWorkflow, FlyteLaunchPlan],
        inputs: typing.Dict[str, typing.Any],
        project: str = None,
        domain: str = None,
        execution_name: typing.Optional[str] = None,
        execution_name_prefix: typing.Optional[str] = None,
        options: typing.Optional[Options] = None,
        wait: bool = False,
        type_hints: typing.Optional[typing.Dict[str, typing.Type]] = None,
        overwrite_cache: typing.Optional[bool] = None,
        envs: typing.Optional[typing.Dict[str, str]] = None,
        tags: typing.Optional[typing.List[str]] = None,
        cluster_pool: typing.Optional[str] = None,
    ) -> FlyteWorkflowExecution:
        resolved_inputs = {}
        for k, v in inputs.items():
            # TODO: Remove when https://github.com/flyteorg/flytekit/pull/2136 gets merged
            if Artifact is None:
                resolved_inputs[k] = v
            elif isinstance(v, artifacts_pb2.Artifact):
                lit = v.spec.value
                resolved_inputs[k] = lit
            elif isinstance(v, Artifact):
                if v.literal is not None:
                    lit = v.literal
                elif v.to_id_idl() is not None:
                    fetched_artifact = self.get_artifact(artifact_id=v.concrete_artifact_id)
                    if not fetched_artifact:
                        raise user_exceptions.FlyteValueException(
                            v.concrete_artifact_id, "Could not find artifact with ID given"
                        )
                    lit = fetched_artifact.literal
                else:
                    raise user_exceptions.FlyteValueException(
                        v, "When binding input to Artifact, either the Literal or the ID must be set"
                    )
                resolved_inputs[k] = lit
            else:
                resolved_inputs[k] = v

        return super()._execute(
            entity,
            resolved_inputs,
            project=project,
            domain=domain,
            execution_name=execution_name,
            execution_name_prefix=execution_name_prefix,
            options=options,
            wait=wait,
            type_hints=type_hints,
            overwrite_cache=overwrite_cache,
            envs=envs,
            tags=tags,
            cluster_pool=cluster_pool,
        )

    def create_artifact(self, artifact: Artifact):
        """
        Create an artifact in FlyteAdmin.

        :param artifact: The artifact to create.
        :return: The artifact as persisted in the service.
        """
        # Two things can happen here -
        #  - the call to to_literal may upload something, in the case of an offloaded data type.
        #    - if this happens, the upload request should already return the created Artifact object.
        if artifact.literal is None:
            with self.remote_context() as ctx:
                lt = artifact.literal_type or TypeEngine.to_literal_type(artifact.python_type)
                lit = TypeEngine.to_literal(ctx, artifact.python_val, artifact.python_type, lt)
                artifact.literal_type = lt.to_flyte_idl()
                artifact.literal = lit.to_flyte_idl()
        else:
            raise ValueError("Cannot create an artifact with a literal already set.")

        # Need to detect here what happened. If the conversion triggered a data upload, then the information from
        # Artifact object from that call should be copied in.
        # If not, an explicit call to create_artifact should be made.

        if artifact.project is None:
            artifact.project = self.default_project
        if artifact.domain is None:
            artifact.domain = self.default_domain
        create_request = Artifact.to_create_request(artifact)
        resp = self.artifacts_client.CreateArtifact(create_request)
        logging.debug(f"CreateArtifact response {resp}")
        artifact.project = resp.artifact.artifact_id.artifact_key.project
        artifact.domain = resp.artifact.artifact_id.artifact_key.domain
        artifact.name = resp.artifact.artifact_id.artifact_key.name
        artifact.version = resp.artifact.artifact_id.version
        artifact.source = (
            resp.artifact.spec.principal or resp.artifact.spec.execution or resp.artifact.spec.task_execution
        )



import base64
import sys
import textwrap
from dataclasses import dataclass
from importlib.metadata import version
from os import environ, getenv
from pathlib import Path
from string import Template
from typing import Optional

from flyteidl.service.identity_pb2 import UserInfoRequest
from flyteidl.service.identity_pb2_grpc import IdentityServiceStub
from flytekit.clients.auth_helper import get_authenticated_channel
from flytekit.configuration import AuthType, Config, ConfigFile, PlatformConfig, get_config_file
from flytekit.configuration.default_images import DefaultImages

_DEFAULT_GCP_SERVERLESS_ENDPOINT: str = "serverless-gcp.cloud-staging.union.ai"
_SERVERLESS_IMAGE_REGISTRY: str = "us-central1-docker.pkg.dev/serverless-gcp-dataplane/union/unionai"

_UNIONAI_CONFIG_ENV_VAR: str = "UNIONAI_CONFIG"
_UNIONAI_SERVERLESS_API_KEY_ENV_VAR: str = "UNIONAI_SERVERLESS_API_KEY"
_UNIONAI_DEFAULT_CONFIG_DIR: Path = Path.home() / ".unionai"
_UNIONAI_CONFIG_NAME: str = "config.yaml"
_UNIONAI_DEFAULT_CONFIG_PATH: Path = _UNIONAI_DEFAULT_CONFIG_DIR / _UNIONAI_CONFIG_NAME


@dataclass
class _UnionAIConfig:
    serverless_endpoint: str = getenv("UNIONAI_SERVERLESS_ENDPOINT", _DEFAULT_GCP_SERVERLESS_ENDPOINT)
    org: Optional[str] = None
    config: Optional[str] = None
    is_direct_unionai_cli_call: bool = False


_UNIONAI_CONFIG = _UnionAIConfig()


def _is_serverless_endpoint(endpoint: str) -> str:
    """Check if endpoint is serverless."""
    serverless_endpoint = _UNIONAI_CONFIG.serverless_endpoint
    return endpoint in (serverless_endpoint, f"dns:///{serverless_endpoint}")


@dataclass
class AppClientCredentials:
    endpoint: str
    client_id: str
    client_secret: str
    org: str


def _encode_app_client_credentials(app_credentials: AppClientCredentials) -> str:
    """Encode app_credentials with base64."""
    data = (
        f"{app_credentials.endpoint}:{app_credentials.client_id}:{app_credentials.client_secret}:{app_credentials.org}"
    )
    return base64.b64encode(data.encode("utf-8")).decode("utf-8")


def _decode_app_client_credentials(encoded_str: str) -> AppClientCredentials:
    """Decode encoded base64 string into app credentials."""
    endpoint, client_id, client_secret, org = base64.b64decode(encoded_str.encode("utf-8")).decode("utf-8").split(":")
    return AppClientCredentials(endpoint=endpoint, client_id=client_id, client_secret=client_secret, org=org)


def _get_config_obj(config: Optional[str] = None, default_to_serverless: bool = False) -> Config:
    """Get Config object."""
    if _UNIONAI_CONFIG.is_direct_unionai_cli_call or default_to_serverless:
        # CLI ran with `unionai` -> Config is _UNIONAI_CONFIG.config and the default is serverless
        if _UNIONAI_CONFIG.config is not None:
            cfg_file = ConfigFile(_UNIONAI_CONFIG.config)
        elif _UNIONAI_DEFAULT_CONFIG_PATH.exists():
            # Read config from file system
            cfg_file = ConfigFile(str(_UNIONAI_DEFAULT_CONFIG_PATH.absolute()))
        else:
            # Setting None here uses serverless
            cfg_file = None
    else:
        # CLI ran with `pyflyte`, use the same semantics as `pyflyte`.
        # If `get_config_file` returns None, then we go to serverless
        if config is None:
            cfg_file = get_config_file(_UNIONAI_CONFIG.config)
        else:
            cfg_file = get_config_file(config)

    if cfg_file is None:
        serverless_api_value = environ.get(_UNIONAI_SERVERLESS_API_KEY_ENV_VAR, "")
        config = Config.for_endpoint(endpoint=_UNIONAI_CONFIG.serverless_endpoint)

        if serverless_api_value == "":
            # Serverless clients points to the serverless endpoint by default.
            return config
        try:
            app_credentials = _decode_app_client_credentials(serverless_api_value)
        except Exception as e:
            raise ValueError(f"Unable to read {_UNIONAI_SERVERLESS_API_KEY_ENV_VAR}") from e

        if _UNIONAI_CONFIG.org is None and app_credentials.org != "":
            _UNIONAI_CONFIG.org = app_credentials.org

        return config.with_params(
            platform=PlatformConfig(
                endpoint=app_credentials.endpoint,
                insecure=False,
                auth_mode=AuthType.CLIENTSECRET,
                client_id=app_credentials.client_id,
                client_credentials_secret=app_credentials.client_secret,
            )
        )
    else:
        # Allow for --config to still be passed in for Managed+ users.
        return Config.auto(cfg_file)


def _get_organization(platform_config: PlatformConfig) -> str:
    """Get organization based on endpoint."""
    if _UNIONAI_CONFIG.org is not None:
        return _UNIONAI_CONFIG.org
    elif _is_serverless_endpoint(platform_config.endpoint):
        org = _get_user_handle(platform_config)
        _UNIONAI_CONFIG.org = org
        return org
    else:
        # Managed+ users, the org is not required for requests and we set it ""
        # to replicate default flytekit behavior.
        return ""


def _get_user_handle(platform_config: PlatformConfig) -> str:
    """Get user_handle for PlatformConfig."""
    with get_authenticated_channel(platform_config) as channel:
        client = IdentityServiceStub(channel)
        user_info = client.UserInfo(UserInfoRequest())
        user_handle = user_info.additional_claims.fields["userhandle"]
        return user_handle.string_value


def _get_default_image() -> str:
    """Get default image version."""
    cfg_obj = _get_config_obj()

    # TODO: This is only temporary to support GCP endpoints. When the unionai images are public,
    # we will always use unionai images
    if _is_serverless_endpoint(cfg_obj.platform.endpoint):
        major, minor = sys.version_info.major, sys.version_info.minor
        unionai_version = version("unionai")
        if "dev" in unionai_version:
            suffix = "latest"
        else:
            suffix = unionai_version

        return f"{_SERVERLESS_IMAGE_REGISTRY}:py{major}.{minor}-{suffix}"

    return DefaultImages().find_image_for()


def _write_config_to_path(endpoint: str, auth_type: str, config_dir: Path = _UNIONAI_DEFAULT_CONFIG_DIR):
    """Write config to config directory."""
    config_dir.mkdir(exist_ok=True, parents=True)

    config_template = Template(
        textwrap.dedent(
            """\
    admin:
      endpoint: $endpoint
      insecure: false
      authType: $auth_type
    logger:
      show-source: true
      level: 0
    union:
      connection:
        host: $endpoint
        insecure: false
      auth:
        type: $auth_type
    """
        )
    )
    config_path = config_dir / _UNIONAI_CONFIG_NAME
    config_path.write_text(config_template.substitute(endpoint=endpoint, auth_type=auth_type))

    return config_path


def _get_default_project(previous_default: str):
    cfg_obj = _get_config_obj()
    if _is_serverless_endpoint(cfg_obj.platform.endpoint):
        return "default"
    return previous_default


def _get_auth_success_html(endpoint: str) -> str:
    """Get default success html. Return None to use flytekit's default success html."""
    if endpoint.endswith("union.ai") or endpoint.endswith("unionai.cloud"):
        SUCCESS_HTML = textwrap.dedent(
            f"""
        <html>
        <head>
            <title>OAuth2 Authentication to UnionAI Successful</title>
        </head>
        <body style="background:white;font-family:Arial">
            <div style="position: absolute;top:40%;left:50%;transform: translate(-50%, -50%);text-align:center;">
                <div style="margin:auto">
                    <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 300 65" fill="currentColor"
                        style="color:#fdb51e;width:360px;">
                        <title>Union.ai</title>
                        <path d="M32,64.8C14.4,64.8,0,51.5,0,34V3.6h17.6v41.3c0,1.9,1.1,3,3,3h23c1.9,0,3-1.1,3-3V3.6H64V34
                        C64,51.5,49.6,64.8,32,64.8z M69.9,30.9v30.4h17.6V20c0-1.9,1.1-3,3-3h23c1.9,0,3,1.1,3,3v41.3H134V30.9c0-17.5-14.4-30.8-32.1-30.8
                        S69.9,13.5,69.9,30.9z M236,30.9v30.4h17.6V20c0-1.9,1.1-3,3-3h23c1.9,0,3,1.1,3,3v41.3H300V30.9c0-17.5-14.4-30.8-32-30.8
                        S236,13.5,236,30.9L236,30.9z M230.1,32.4c0,18.2-14.2,32.5-32.2,32.5s-32-14.3-32-32.5s14-32.1,32-32.1S230.1,14.3,230.1,32.4
                        L230.1,32.4z M213.5,20.2c0-1.9-1.1-3-3-3h-24.8c-1.9,0-3,1.1-3,3v24.5c0,1.9,1.1,3,3,3h24.8c1.9,0,3-1.1,3-3V20.2z M158.9,3.6
                        h-17.6v57.8h17.6V3.6z"></path>
                    </svg>
                    <h2>You've successfully authenticated to:
                        <br><em>{endpoint}</em>
                    </h2>
                    <p style="font-size:20px;">Return to your terminal for next steps</p>
                </div>
            </div>
        </body>
        </html>
        """  # noqa
        )
        return SUCCESS_HTML
    return None



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: objectstore/definition.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from unionai.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cobjectstore/definition.proto\x12\x17\x63loudidl.objectstore.v1\x1a\x17validate/validate.proto\"\x80\x01\n\x08Metadata\x12<\n\x03tag\x18\x02 \x03(\x0b\x32*.cloudidl.objectstore.v1.Metadata.TagEntryR\x03tag\x1a\x36\n\x08TagEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"$\n\x06Object\x12\x1a\n\x08\x63ontents\x18\x01 \x01(\x0cR\x08\x63ontents\"4\n\x03Key\x12-\n\x03key\x18\x01 \x01(\tB\x1b\xfa\x42\x18r\x16\x10\x01\x32\x12^union(meta)?://.+R\x03key\"4\n\x11PresignPutRequest\x12\x1f\n\x0b\x63ontent_md5\x18\x01 \x01(\x0cR\ncontentMd5\"\x13\n\x11PresignGetRequest*/\n\nHTTPMethod\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x07\n\x03GET\x10\x01\x12\x07\n\x03PUT\x10\x02\x42\xde\x01\n\x1b\x63om.cloudidl.objectstore.v1B\x0f\x44\x65\x66initionProtoH\x02P\x01Z.github.com/unionai/cloud/gen/pb-go/objectstore\xa2\x02\x03\x43OX\xaa\x02\x17\x43loudidl.Objectstore.V1\xca\x02\x17\x43loudidl\\Objectstore\\V1\xe2\x02#Cloudidl\\Objectstore\\V1\\GPBMetadata\xea\x02\x19\x43loudidl::Objectstore::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'objectstore.definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.cloudidl.objectstore.v1B\017DefinitionProtoH\002P\001Z.github.com/unionai/cloud/gen/pb-go/objectstore\242\002\003COX\252\002\027Cloudidl.Objectstore.V1\312\002\027Cloudidl\\Objectstore\\V1\342\002#Cloudidl\\Objectstore\\V1\\GPBMetadata\352\002\031Cloudidl::Objectstore::V1'
  _METADATA_TAGENTRY._options = None
  _METADATA_TAGENTRY._serialized_options = b'8\001'
  _KEY.fields_by_name['key']._options = None
  _KEY.fields_by_name['key']._serialized_options = b'\372B\030r\026\020\0012\022^union(meta)?://.+'
  _globals['_HTTPMETHOD']._serialized_start=380
  _globals['_HTTPMETHOD']._serialized_end=427
  _globals['_METADATA']._serialized_start=83
  _globals['_METADATA']._serialized_end=211
  _globals['_METADATA_TAGENTRY']._serialized_start=157
  _globals['_METADATA_TAGENTRY']._serialized_end=211
  _globals['_OBJECT']._serialized_start=213
  _globals['_OBJECT']._serialized_end=249
  _globals['_KEY']._serialized_start=251
  _globals['_KEY']._serialized_end=303
  _globals['_PRESIGNPUTREQUEST']._serialized_start=305
  _globals['_PRESIGNPUTREQUEST']._serialized_end=357
  _globals['_PRESIGNGETREQUEST']._serialized_start=359
  _globals['_PRESIGNGETREQUEST']._serialized_end=378
# @@protoc_insertion_point(module_scope)



from google.protobuf import struct_pb2 as _struct_pb2
from google.api import annotations_pb2 as _annotations_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from flyteidl.admin import launch_plan_pb2 as _launch_plan_pb2
from flyteidl.core import literals_pb2 as _literals_pb2
from flyteidl.core import types_pb2 as _types_pb2
from flyteidl.core import identifier_pb2 as _identifier_pb2
from flyteidl.core import artifact_id_pb2 as _artifact_id_pb2
from flyteidl.core import interface_pb2 as _interface_pb2
from flyteidl.event import cloudevents_pb2 as _cloudevents_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Artifact(_message.Message):
    __slots__ = ["artifact_id", "spec", "tags", "source", "metadata"]
    ARTIFACT_ID_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    artifact_id: _artifact_id_pb2.ArtifactID
    spec: ArtifactSpec
    tags: _containers.RepeatedScalarFieldContainer[str]
    source: ArtifactSource
    metadata: ArtifactMetadata
    def __init__(self, artifact_id: _Optional[_Union[_artifact_id_pb2.ArtifactID, _Mapping]] = ..., spec: _Optional[_Union[ArtifactSpec, _Mapping]] = ..., tags: _Optional[_Iterable[str]] = ..., source: _Optional[_Union[ArtifactSource, _Mapping]] = ..., metadata: _Optional[_Union[ArtifactMetadata, _Mapping]] = ...) -> None: ...

class ArtifactMetadata(_message.Message):
    __slots__ = ["created_at", "uri"]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    URI_FIELD_NUMBER: _ClassVar[int]
    created_at: _timestamp_pb2.Timestamp
    uri: str
    def __init__(self, created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., uri: _Optional[str] = ...) -> None: ...

class CreateArtifactRequest(_message.Message):
    __slots__ = ["artifact_key", "version", "spec", "partitions", "time_partition_value", "source"]
    class PartitionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ARTIFACT_KEY_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    TIME_PARTITION_VALUE_FIELD_NUMBER: _ClassVar[int]
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    artifact_key: _artifact_id_pb2.ArtifactKey
    version: str
    spec: ArtifactSpec
    partitions: _containers.ScalarMap[str, str]
    time_partition_value: _timestamp_pb2.Timestamp
    source: ArtifactSource
    def __init__(self, artifact_key: _Optional[_Union[_artifact_id_pb2.ArtifactKey, _Mapping]] = ..., version: _Optional[str] = ..., spec: _Optional[_Union[ArtifactSpec, _Mapping]] = ..., partitions: _Optional[_Mapping[str, str]] = ..., time_partition_value: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., source: _Optional[_Union[ArtifactSource, _Mapping]] = ...) -> None: ...

class ArtifactSource(_message.Message):
    __slots__ = ["workflow_execution", "node_id", "task_id", "retry_attempt", "principal"]
    WORKFLOW_EXECUTION_FIELD_NUMBER: _ClassVar[int]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    RETRY_ATTEMPT_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    workflow_execution: _identifier_pb2.WorkflowExecutionIdentifier
    node_id: str
    task_id: _identifier_pb2.Identifier
    retry_attempt: int
    principal: str
    def __init__(self, workflow_execution: _Optional[_Union[_identifier_pb2.WorkflowExecutionIdentifier, _Mapping]] = ..., node_id: _Optional[str] = ..., task_id: _Optional[_Union[_identifier_pb2.Identifier, _Mapping]] = ..., retry_attempt: _Optional[int] = ..., principal: _Optional[str] = ...) -> None: ...

class Card(_message.Message):
    __slots__ = ["uri", "type", "body", "text_type"]
    class CardType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNKNOWN: _ClassVar[Card.CardType]
        MODEL: _ClassVar[Card.CardType]
        DATASET: _ClassVar[Card.CardType]
    UNKNOWN: Card.CardType
    MODEL: Card.CardType
    DATASET: Card.CardType
    class TextType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        MARKDOWN: _ClassVar[Card.TextType]
    MARKDOWN: Card.TextType
    URI_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    BODY_FIELD_NUMBER: _ClassVar[int]
    TEXT_TYPE_FIELD_NUMBER: _ClassVar[int]
    uri: str
    type: Card.CardType
    body: str
    text_type: Card.TextType
    def __init__(self, uri: _Optional[str] = ..., type: _Optional[_Union[Card.CardType, str]] = ..., body: _Optional[str] = ..., text_type: _Optional[_Union[Card.TextType, str]] = ...) -> None: ...

class ArtifactSpec(_message.Message):
    __slots__ = ["value", "type", "short_description", "user_metadata", "card"]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    SHORT_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    USER_METADATA_FIELD_NUMBER: _ClassVar[int]
    CARD_FIELD_NUMBER: _ClassVar[int]
    value: _literals_pb2.Literal
    type: _types_pb2.LiteralType
    short_description: str
    user_metadata: _struct_pb2.Struct
    card: Card
    def __init__(self, value: _Optional[_Union[_literals_pb2.Literal, _Mapping]] = ..., type: _Optional[_Union[_types_pb2.LiteralType, _Mapping]] = ..., short_description: _Optional[str] = ..., user_metadata: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., card: _Optional[_Union[Card, _Mapping]] = ...) -> None: ...

class Trigger(_message.Message):
    __slots__ = ["trigger", "trigger_inputs"]
    TRIGGER_FIELD_NUMBER: _ClassVar[int]
    TRIGGER_INPUTS_FIELD_NUMBER: _ClassVar[int]
    trigger: _artifact_id_pb2.ArtifactID
    trigger_inputs: _interface_pb2.ParameterMap
    def __init__(self, trigger: _Optional[_Union[_artifact_id_pb2.ArtifactID, _Mapping]] = ..., trigger_inputs: _Optional[_Union[_interface_pb2.ParameterMap, _Mapping]] = ...) -> None: ...

class CreateArtifactResponse(_message.Message):
    __slots__ = ["artifact"]
    ARTIFACT_FIELD_NUMBER: _ClassVar[int]
    artifact: Artifact
    def __init__(self, artifact: _Optional[_Union[Artifact, _Mapping]] = ...) -> None: ...

class GetArtifactRequest(_message.Message):
    __slots__ = ["query", "details"]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    DETAILS_FIELD_NUMBER: _ClassVar[int]
    query: _artifact_id_pb2.ArtifactQuery
    details: bool
    def __init__(self, query: _Optional[_Union[_artifact_id_pb2.ArtifactQuery, _Mapping]] = ..., details: bool = ...) -> None: ...

class GetArtifactResponse(_message.Message):
    __slots__ = ["artifact"]
    ARTIFACT_FIELD_NUMBER: _ClassVar[int]
    artifact: Artifact
    def __init__(self, artifact: _Optional[_Union[Artifact, _Mapping]] = ...) -> None: ...

class SearchOptions(_message.Message):
    __slots__ = ["strict_partitions", "latest_by_key"]
    STRICT_PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    LATEST_BY_KEY_FIELD_NUMBER: _ClassVar[int]
    strict_partitions: bool
    latest_by_key: bool
    def __init__(self, strict_partitions: bool = ..., latest_by_key: bool = ...) -> None: ...

class SearchArtifactsRequest(_message.Message):
    __slots__ = ["artifact_key", "partitions", "time_partition_value", "principal", "version", "options", "token", "limit", "granularity"]
    ARTIFACT_KEY_FIELD_NUMBER: _ClassVar[int]
    PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    TIME_PARTITION_VALUE_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    GRANULARITY_FIELD_NUMBER: _ClassVar[int]
    artifact_key: _artifact_id_pb2.ArtifactKey
    partitions: _artifact_id_pb2.Partitions
    time_partition_value: _timestamp_pb2.Timestamp
    principal: str
    version: str
    options: SearchOptions
    token: str
    limit: int
    granularity: _artifact_id_pb2.Granularity
    def __init__(self, artifact_key: _Optional[_Union[_artifact_id_pb2.ArtifactKey, _Mapping]] = ..., partitions: _Optional[_Union[_artifact_id_pb2.Partitions, _Mapping]] = ..., time_partition_value: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., principal: _Optional[str] = ..., version: _Optional[str] = ..., options: _Optional[_Union[SearchOptions, _Mapping]] = ..., token: _Optional[str] = ..., limit: _Optional[int] = ..., granularity: _Optional[_Union[_artifact_id_pb2.Granularity, str]] = ...) -> None: ...

class SearchArtifactsResponse(_message.Message):
    __slots__ = ["artifacts", "token"]
    ARTIFACTS_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    artifacts: _containers.RepeatedCompositeFieldContainer[Artifact]
    token: str
    def __init__(self, artifacts: _Optional[_Iterable[_Union[Artifact, _Mapping]]] = ..., token: _Optional[str] = ...) -> None: ...

class FindByWorkflowExecRequest(_message.Message):
    __slots__ = ["exec_id", "direction"]
    class Direction(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        INPUTS: _ClassVar[FindByWorkflowExecRequest.Direction]
        OUTPUTS: _ClassVar[FindByWorkflowExecRequest.Direction]
    INPUTS: FindByWorkflowExecRequest.Direction
    OUTPUTS: FindByWorkflowExecRequest.Direction
    EXEC_ID_FIELD_NUMBER: _ClassVar[int]
    DIRECTION_FIELD_NUMBER: _ClassVar[int]
    exec_id: _identifier_pb2.WorkflowExecutionIdentifier
    direction: FindByWorkflowExecRequest.Direction
    def __init__(self, exec_id: _Optional[_Union[_identifier_pb2.WorkflowExecutionIdentifier, _Mapping]] = ..., direction: _Optional[_Union[FindByWorkflowExecRequest.Direction, str]] = ...) -> None: ...

class AddTagRequest(_message.Message):
    __slots__ = ["artifact_id", "value", "overwrite"]
    ARTIFACT_ID_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    OVERWRITE_FIELD_NUMBER: _ClassVar[int]
    artifact_id: _artifact_id_pb2.ArtifactID
    value: str
    overwrite: bool
    def __init__(self, artifact_id: _Optional[_Union[_artifact_id_pb2.ArtifactID, _Mapping]] = ..., value: _Optional[str] = ..., overwrite: bool = ...) -> None: ...

class AddTagResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ActivateTriggerRequest(_message.Message):
    __slots__ = ["trigger_id"]
    TRIGGER_ID_FIELD_NUMBER: _ClassVar[int]
    trigger_id: _identifier_pb2.Identifier
    def __init__(self, trigger_id: _Optional[_Union[_identifier_pb2.Identifier, _Mapping]] = ...) -> None: ...

class ActivateTriggerResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class CreateTriggerRequest(_message.Message):
    __slots__ = ["trigger_launch_plan"]
    TRIGGER_LAUNCH_PLAN_FIELD_NUMBER: _ClassVar[int]
    trigger_launch_plan: _launch_plan_pb2.LaunchPlan
    def __init__(self, trigger_launch_plan: _Optional[_Union[_launch_plan_pb2.LaunchPlan, _Mapping]] = ...) -> None: ...

class CreateTriggerResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class DeactivateTriggerRequest(_message.Message):
    __slots__ = ["trigger_id"]
    TRIGGER_ID_FIELD_NUMBER: _ClassVar[int]
    trigger_id: _identifier_pb2.Identifier
    def __init__(self, trigger_id: _Optional[_Union[_identifier_pb2.Identifier, _Mapping]] = ...) -> None: ...

class DeactivateTriggerResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class DeactivateAllTriggersRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class DeactivateAllTriggersResponse(_message.Message):
    __slots__ = ["num_deactivated"]
    NUM_DEACTIVATED_FIELD_NUMBER: _ClassVar[int]
    num_deactivated: int
    def __init__(self, num_deactivated: _Optional[int] = ...) -> None: ...

class ArtifactProducer(_message.Message):
    __slots__ = ["entity_id", "outputs"]
    ENTITY_ID_FIELD_NUMBER: _ClassVar[int]
    OUTPUTS_FIELD_NUMBER: _ClassVar[int]
    entity_id: _identifier_pb2.Identifier
    outputs: _interface_pb2.VariableMap
    def __init__(self, entity_id: _Optional[_Union[_identifier_pb2.Identifier, _Mapping]] = ..., outputs: _Optional[_Union[_interface_pb2.VariableMap, _Mapping]] = ...) -> None: ...

class RegisterProducerRequest(_message.Message):
    __slots__ = ["producers"]
    PRODUCERS_FIELD_NUMBER: _ClassVar[int]
    producers: _containers.RepeatedCompositeFieldContainer[ArtifactProducer]
    def __init__(self, producers: _Optional[_Iterable[_Union[ArtifactProducer, _Mapping]]] = ...) -> None: ...

class ArtifactConsumer(_message.Message):
    __slots__ = ["entity_id", "inputs"]
    ENTITY_ID_FIELD_NUMBER: _ClassVar[int]
    INPUTS_FIELD_NUMBER: _ClassVar[int]
    entity_id: _identifier_pb2.Identifier
    inputs: _interface_pb2.ParameterMap
    def __init__(self, entity_id: _Optional[_Union[_identifier_pb2.Identifier, _Mapping]] = ..., inputs: _Optional[_Union[_interface_pb2.ParameterMap, _Mapping]] = ...) -> None: ...

class RegisterConsumerRequest(_message.Message):
    __slots__ = ["consumers"]
    CONSUMERS_FIELD_NUMBER: _ClassVar[int]
    consumers: _containers.RepeatedCompositeFieldContainer[ArtifactConsumer]
    def __init__(self, consumers: _Optional[_Iterable[_Union[ArtifactConsumer, _Mapping]]] = ...) -> None: ...

class RegisterResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ExecutionInputsRequest(_message.Message):
    __slots__ = ["execution_id", "inputs"]
    EXECUTION_ID_FIELD_NUMBER: _ClassVar[int]
    INPUTS_FIELD_NUMBER: _ClassVar[int]
    execution_id: _identifier_pb2.WorkflowExecutionIdentifier
    inputs: _containers.RepeatedCompositeFieldContainer[_artifact_id_pb2.ArtifactID]
    def __init__(self, execution_id: _Optional[_Union[_identifier_pb2.WorkflowExecutionIdentifier, _Mapping]] = ..., inputs: _Optional[_Iterable[_Union[_artifact_id_pb2.ArtifactID, _Mapping]]] = ...) -> None: ...

class ExecutionInputsResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListUsageRequest(_message.Message):
    __slots__ = ["artifact_id"]
    ARTIFACT_ID_FIELD_NUMBER: _ClassVar[int]
    artifact_id: _artifact_id_pb2.ArtifactID
    def __init__(self, artifact_id: _Optional[_Union[_artifact_id_pb2.ArtifactID, _Mapping]] = ...) -> None: ...

class ListUsageResponse(_message.Message):
    __slots__ = ["executions"]
    EXECUTIONS_FIELD_NUMBER: _ClassVar[int]
    executions: _containers.RepeatedCompositeFieldContainer[_identifier_pb2.WorkflowExecutionIdentifier]
    def __init__(self, executions: _Optional[_Iterable[_Union[_identifier_pb2.WorkflowExecutionIdentifier, _Mapping]]] = ...) -> None: ...

class GetCardRequest(_message.Message):
    __slots__ = ["artifact_id"]
    ARTIFACT_ID_FIELD_NUMBER: _ClassVar[int]
    artifact_id: _artifact_id_pb2.ArtifactID
    def __init__(self, artifact_id: _Optional[_Union[_artifact_id_pb2.ArtifactID, _Mapping]] = ...) -> None: ...

class GetCardResponse(_message.Message):
    __slots__ = ["card"]
    CARD_FIELD_NUMBER: _ClassVar[int]
    card: Card
    def __init__(self, card: _Optional[_Union[Card, _Mapping]] = ...) -> None: ...



from typing import Optional

import rich_click as click
from grpc import RpcError

from unionai._config import _UNIONAI_DEFAULT_CONFIG_PATH, _get_config_obj, _get_organization
from unionai._interceptor import get_authenticated_channel_with_org
from unionai.internal.identity.app_payload_pb2 import DeleteAppRequest, GetAppRequest
from unionai.internal.identity.app_service_pb2_grpc import AppsServiceStub
from unionai.internal.secret.definition_pb2 import SecretIdentifier
from unionai.internal.secret.payload_pb2 import DeleteSecretRequest
from unionai.internal.secret.secret_pb2_grpc import SecretServiceStub


@click.group()
def delete():
    """Delete a resource."""


@delete.command()
@click.pass_context
@click.argument("name")
@click.option("--project", help="Project name")
@click.option("--domain", help="Domain name")
def secret(
    ctx: click.Context,
    name: str,
    project: Optional[str],
    domain: Optional[str],
):
    """Delete secret with NAME."""
    platform_obj = _get_config_obj(ctx.obj.get("config_file", None)).platform
    org = _get_organization(platform_obj)

    channel = get_authenticated_channel_with_org(platform_obj, org)
    stub = SecretServiceStub(channel)

    request = DeleteSecretRequest(
        id=SecretIdentifier(name=name, domain=domain, project=project, organization=org),
    )
    try:
        stub.DeleteSecret(request)
        click.echo(f"Deleted secret with name: {name}")
    except RpcError as e:
        raise click.ClickException(f"Unable to delete secret with name: {name}\n{e}") from e


@delete.command()
@click.pass_context
@click.argument("client_id")
def app(
    ctx: click.Context,
    client_id: str,
):
    """Delete app with CLIENT_ID."""
    platform_obj = _get_config_obj(ctx.obj.get("config_file", None)).platform
    org = _get_organization(platform_obj)

    channel = get_authenticated_channel_with_org(platform_obj, org)
    stub = AppsServiceStub(channel)

    # Get app first to make sure it exist
    get_request = GetAppRequest(organization=org, client_id=client_id)
    try:
        stub.Get(get_request)
    except RpcError as e:
        raise click.ClickException(
            f"Unable to delete app with client_id: {client_id} because it does not exist.\n{e}"
        ) from e

    delete_request = DeleteAppRequest(organization=org, client_id=client_id)
    try:
        stub.Delete(delete_request)
        click.echo(f"Deleted app with client_id: {client_id}")
    except RpcError as e:
        raise click.ClickException(f"Unable to delete app with client_id: {client_id}\n{e}") from e


@delete.command()
def login():
    """Delete login information."""
    if not _UNIONAI_DEFAULT_CONFIG_PATH.exists():
        click.echo(f"Login at {_UNIONAI_DEFAULT_CONFIG_PATH} does not exist. No need to delete.")
        return

    try:
        _UNIONAI_DEFAULT_CONFIG_PATH.unlink()
        click.echo("Deleted login.")
    except Exception as e:
        msg = f"Unable to delete login.\n{e}"
        raise click.ClickException(msg) from e



"""All imports to unionai in this file should be in the function definition.

This plugin is loaded by flytekit, so any imports to unionai can lead to circular imports.
"""

from typing import Optional

import click
from click import Group, Option
from flytekit.remote import FlyteRemote


class UnionAIPlugin:
    @staticmethod
    def get_remote(
        config: Optional[str],
        project: str,
        domain: str,
        data_upload_location: Optional[str] = None,
    ) -> FlyteRemote:
        from unionai._config import _get_config_obj, _is_serverless_endpoint
        from unionai.remote import UnionRemote

        cfg_obj = _get_config_obj(config)
        if _is_serverless_endpoint(cfg_obj.platform.endpoint) and project == "flytesnacks":
            project = "default"

        return UnionRemote(
            cfg_obj,
            default_project=project,
            default_domain=domain,
            data_upload_location=data_upload_location,
        )

    @staticmethod
    def configure_pyflyte_cli(main: Group) -> Group:
        """Configure pyflyte's CLI."""

        import unionai._config
        from unionai._config import _UNIONAI_CONFIG, _get_config_obj
        from unionai.cli._create import create
        from unionai.cli._delete import delete
        from unionai.cli._get import get
        from unionai.cli._update import update
        from unionai.ucimage._image_builder import _register_union_image_builder

        def _cli_main_config_callback(ctx, param, value):
            # Set org based on config from `pyflyte --config` for downstream cli
            # commands to use.
            if value is not None:
                _UNIONAI_CONFIG.config = value

            # Only register image builder for serverless
            cfg_obj = _get_config_obj(value)
            if unionai._config._is_serverless_endpoint(cfg_obj.platform.endpoint):
                _register_union_image_builder()
            return value

        for p in main.params:
            if p.name == "config":
                p.callback = _cli_main_config_callback

        # Configure org at the top level:
        def _set_org(ctx, param, value):
            _UNIONAI_CONFIG.org = value

        main.params.append(
            click.Option(
                ["--org"],
                help="Set organization",
                hidden=True,
                callback=_set_org,
                expose_value=False,
            )
        )

        from flytekit.clis.sdk_in_container.run import RunCommand, RunLevelParams, run

        class RunWithNewProjectDefault(RunCommand):
            def __init__(self, *args, **kwargs) -> None:
                super().__init__(*args, **kwargs)
                for p in self.params:
                    if p.name == "image_config":
                        p.default = lambda: [unionai._config._get_default_image()]

            def get_command(self, ctx, filename):
                command = super().get_command(ctx, filename)
                run_params: RunLevelParams = ctx.obj
                if run_params.is_remote and run_params.project == "flytesnacks":
                    run_params.project = unionai._config._get_default_project(run_params.project)
                return command

        main.add_command(RunWithNewProjectDefault(name="run", help=run.help), "run")

        def update_or_create_group(new_group):
            try:
                main_group = main.commands[new_group.name]
                for name, command in new_group.commands.items():
                    main_group.add_command(command, name)
            except KeyError:
                main.add_command(new_group)

        new_groups = [create, delete, update, get]
        for group in new_groups:
            update_or_create_group(group)

        def yield_all_options():
            commands = [main]

            while commands:
                command = commands.pop()
                yield from (p for p in command.params if isinstance(p, Option))

                if isinstance(command, Group):
                    commands.extend(list(command.commands.values()))

        for p in yield_all_options():
            if p.name == "image_config" and p.default is not None:
                p.default = lambda: [unionai._config._get_default_image()]
            elif p.name == "project" and p.default is not None:
                p.default = lambda: unionai._config._get_default_project(p.default)

        return main

    @staticmethod
    def secret_requires_group() -> bool:
        """Return True if secrets require group entry."""
        return False

    @staticmethod
    def get_default_image() -> Optional[str]:
        """Return default image."""

        from unionai._config import _get_default_image

        return _get_default_image()

    @staticmethod
    def get_auth_success_html(endpoint: str) -> Optional[str]:
        """Get default success html. Return None to use flytekit's default success html."""
        from unionai._config import _get_auth_success_html

        return _get_auth_success_html(endpoint)



from typing import Optional

import rich_click as click
from grpc import RpcError

from unionai._config import _get_config_obj, _get_organization
from unionai._interceptor import get_authenticated_channel_with_org
from unionai.internal.secret.definition_pb2 import SecretIdentifier, SecretSpec
from unionai.internal.secret.payload_pb2 import UpdateSecretRequest
from unionai.internal.secret.secret_pb2_grpc import SecretServiceStub


@click.group()
def update():
    """Update a resource."""


@update.command()
@click.pass_context
@click.argument("name")
@click.option("--value", prompt="Enter secret value", help="Secret value", hide_input=True)
@click.option("--project", help="Project name")
@click.option("--domain", help="Domain name")
def secret(
    ctx: click.Context,
    name: str,
    value: str,
    project: Optional[str],
    domain: Optional[str],
):
    """Update secret with NAME."""
    platform_obj = _get_config_obj(ctx.obj.get("config_file", None)).platform
    org = _get_organization(platform_obj)

    channel = get_authenticated_channel_with_org(platform_obj, org)
    stub = SecretServiceStub(channel)
    request = UpdateSecretRequest(
        id=SecretIdentifier(name=name, domain=domain, project=project, organization=org),
        secret_spec=SecretSpec(value=value),
    )

    try:
        stub.UpdateSecret(request)
        click.echo(f"Updated secret with name: {name}")
    except RpcError as e:
        raise click.ClickException(f"Unable to update secret with name: {name}\n{e}") from e



"""All imports to unionai in this file should be in the function definition.

This plugin is loaded by flytekit, so any imports to unionai can lead to circular imports.
"""

import json
import shutil
import tempfile
import warnings
from datetime import datetime, timedelta, timezone
from pathlib import Path
from typing import Optional, Tuple

import click
from flytekit.core.constants import REQUIREMENTS_FILE_NAME
from flytekit.image_spec.image_spec import _F_IMG_ID, ImageBuildEngine, ImageSpec, ImageSpecBuilder
from flytekit.models.core import execution as core_execution_models


def _build(spec: Path, context: Optional[Path], target_image: str) -> str:
    """Build image using UnionAI."""
    from unionai._config import _is_serverless_endpoint
    from unionai.configuration._plugin import UnionAIPlugin

    remote = UnionAIPlugin.get_remote(
        config=None,
        project="default",
        domain="development",
    )
    if not _is_serverless_endpoint(remote.config.platform.endpoint):
        raise ValueError(
            "The UnionAI image builder requires a Serverless endpoint. "
            f"Your current endpoint is {remote.config.platform.endpoint}"
        )

    context_url = "" if context is None else remote.upload_file(context)[1]
    spec_url = remote.upload_file(spec)[1]

    entity = remote.fetch_task(project="system", domain="production", name="build-image")

    start = datetime.now(timezone.utc)
    execution = remote.execute(
        entity,
        project="system",
        domain="development",
        inputs={"spec": spec_url, "context": context_url, "target_image": target_image},
    )

    console_url = remote.generate_console_url(execution)

    click.secho("👍 Build submitted!", bold=True, fg="yellow")
    click.secho(
        "⏳ Waiting for build to finish at: " + click.style(console_url, fg="cyan"),
        bold=True,
    )
    execution = remote.wait(execution, poll_interval=timedelta(seconds=1))

    elapsed = str(datetime.now(timezone.utc) - start).split(".")[0]
    if execution.closure.phase == core_execution_models.WorkflowExecutionPhase.SUCCEEDED:
        click.secho(f"✅ Build completed in {elapsed}!", bold=True, fg="green")
    else:
        click.secho(f"❌ Build failed in ! {elapsed}", bold=True, fg="red")
        raise RuntimeError("image build failed")
    return execution.outputs["fully_qualified_image"]


class UCImageSpecBuilder(ImageSpecBuilder):
    """ImageSpec builder for UnionAI."""

    _SUPPORTED_IMAGE_SPEC_PARAMETERS = {
        "name",
        "builder",
        "source_root",
        "env",
        "packages",
        "requirements",
        "apt_packages",
        "cuda",
        "cudnn",
        "platform",
        "pip_index",
        "commands",
    }

    def build_image(self, image_spec: ImageSpec):
        """Build image using UnionAI."""
        image_name = image_spec.image_name()

        with tempfile.TemporaryDirectory() as tmpdir:
            tmp_path = Path(tmpdir)
            spec_path, archive_path = self._validate_configuration(image_spec, tmp_path, image_name)

            fully_qualified_image = _build(spec_path, archive_path, image_name)
            if fully_qualified_image.startswith("cr.union.ai/"):
                # TODO: This replacement needs to happen in the mutating webhook.
                return fully_qualified_image.replace(
                    "cr.union.ai/",
                    "us-central1-docker.pkg.dev/serverless-gcp-dataplane/orgs/",
                )
            return fully_qualified_image

    def _validate_configuration(
        self, image_spec: ImageSpec, tmp_path: Path, image_name: str
    ) -> Tuple[Path, Optional[Path]]:
        """Validate and write configuration for builder."""
        unsupported_parameters = [
            name
            for name, value in vars(image_spec).items()
            if value is not None and name not in self._SUPPORTED_IMAGE_SPEC_PARAMETERS and not name.startswith("_")
        ]
        if unsupported_parameters:
            msg = f"The following parameters are unsupported and ignored: {unsupported_parameters}"
            warnings.warn(msg, UserWarning)

        # Hardcoded for now since our base image only supports 3.11
        spec = {"python_version": "3.11"}

        # Transform image spec into a spec we expect
        if image_spec.apt_packages:
            spec["apt_packages"] = image_spec.apt_packages
        if image_spec.commands:
            spec["commands"] = image_spec.commands
        if image_spec.cuda or image_spec.cudnn:
            spec["enable_gpu"] = True

        env = image_spec.env or {}
        env = {**{_F_IMG_ID: image_name}, **env}
        if env:
            spec["env"] = env
        packages = ["flytekit"]
        if image_spec.packages:
            packages.extend(image_spec.packages)
        spec["python_packages"] = packages
        if image_spec.pip_index:
            spec["pip_extra_index_urls"] = [image_spec.pip_index]

        context_path = tmp_path / "build.uc-image-builder"
        context_path.mkdir(exist_ok=True)

        if image_spec.requirements:
            shutil.copy2(image_spec.requirements, context_path / REQUIREMENTS_FILE_NAME)
            spec["python_requirements_files"] = [REQUIREMENTS_FILE_NAME]

        if image_spec.source_root:
            # Easter egg
            # Load in additional packages before installing pip/apt packages
            vendor_path = Path(image_spec.source_root) / ".vendor"
            if vendor_path.is_dir():
                spec["dist_dirpath"] = ".vendor"
            shutil.copytree(image_spec.source_root, context_path, dirs_exist_ok=True)

        if any(context_path.iterdir()):
            archive_path = Path(shutil.make_archive(tmp_path / "context", "xztar", context_path))
        else:
            archive_path = None

        spec_path = tmp_path / "spec.json"
        with spec_path.open("w") as f:
            json.dump(spec, f)

        return (spec_path, archive_path)


def _register_union_image_builder():
    ImageBuildEngine.register("ucimage", UCImageSpecBuilder(), priority=10)



from typing import Optional

import rich_click as click
from grpc import RpcError
from rich.console import Console
from rich.table import Table

from unionai._config import _get_config_obj, _get_organization
from unionai._interceptor import get_authenticated_channel_with_org
from unionai.internal.common.list_pb2 import ListRequest
from unionai.internal.identity.app_payload_pb2 import ListAppsRequest
from unionai.internal.identity.app_service_pb2_grpc import AppsServiceStub
from unionai.internal.secret.payload_pb2 import ListSecretsRequest
from unionai.internal.secret.secret_pb2_grpc import SecretServiceStub


@click.group()
def get():
    """Get a resource."""


@get.command()
@click.pass_context
@click.option("--project", help="Project name")
@click.option("--domain", help="Domain name")
def secret(
    ctx: click.Context,
    project: Optional[str],
    domain: Optional[str],
):
    """Get secrets."""
    platform_obj = _get_config_obj(ctx.obj.get("config_file", None)).platform
    org = _get_organization(platform_obj)

    channel = get_authenticated_channel_with_org(platform_obj, org)
    stub = SecretServiceStub(channel)
    secrets = []
    next_token, has_next = "", True

    try:
        while has_next:
            request = ListSecretsRequest(domain=domain, project=project, limit=20, token=next_token, organization=org)
            response = stub.ListSecrets(request)
            next_token = response.token
            has_next = next_token != ""

            secrets.extend(response.secrets)
    except RpcError as e:
        raise click.ClickException(f"Unable to get secrets.\n{e}") from e

    if secrets:
        table = Table()
        for name in ["name", "project", "domain"]:
            table.add_column(name, justify="right")

        for secret in secrets:
            project = secret.id.project or "-"
            domain = secret.id.domain or "-"
            table.add_row(secret.id.name, project, domain)

        console = Console()
        console.print(table)
    else:
        click.echo("No secrets found")


@get.command()
@click.pass_context
def app(ctx: click.Context):
    """Get applications."""
    platform_obj = _get_config_obj(ctx.obj.get("config_file", None)).platform
    org = _get_organization(platform_obj)

    channel = get_authenticated_channel_with_org(platform_obj, org)
    stub = AppsServiceStub(channel)

    apps = []
    next_token, has_next = "", True
    try:
        while has_next:
            request = ListAppsRequest(organization=org, request=ListRequest(limit=20, token=next_token))
            response = stub.List(request)
            next_token = response.token
            has_next = next_token != ""

            apps.extend(response.apps)
    except RpcError as e:
        raise click.ClickException(f"Unable to get apps.\n{e}") from e

    if apps:
        table = Table()
        for name in ["client_id", "client_name"]:
            table.add_column(name, overflow="fold")

        for app in apps:
            table.add_row(app.client_id, app.client_name)

        console = Console()
        console.print(table)
    else:
        click.echo("No apps found.")
